$Ontext
 Reseau d echangeurs de chaleur

 D'apres:
 Yee, T F, and Grossmann, I E, Simultaneous Optimization of Models for
 Heat Integration - Heat Exchanger Network Synthesis. Computers and
 Chemical Engineering 14, 10 (1990), 1151-1184.
$Offtext

sets
     i indice des courants chauds /1*2/
     j indice des courants froids /1*2/
     k indice des etapes /1*2/;

scalars
     netape nombre d etape /2/
     dtmin difference minimale de temperature /10/
     dtmax borne superieur pour dt /10000/
     teaue temperature d entree de l tutlite froide /300/
     teaus temperature de sortie de l utilite froide /320/
     tvap temperature de la vapeur /680/
     couteau cout de l utilite froide /15/
     coutvap cout de l utilite chaude /80/
     coutfixe cout fixe d un echangeur /5500/
     coutaire cout d investissement d un echangeur /150/
     heau coeff de trans pour l utilite froide /1/
     hvap coeff de trans pour l utilite chaude /5/;

parameters
          tce(i) temperature d entree du courant chaud    /1 650, 2 590/
          tcs(i) temperature de sortie du courant chaud   /1 370, 2 370/
          tfe(j) temperature d entree du courant froid    /1 410, 2 350/
          tfs(j) temperature de sortie du courant froid   /1 650, 2 500/
          fcpc(i) produit  (FCp) pour les courants chauds /1  10, 2  20/
          fcpf(j) produit  (FCp) pour les courants froids /1  15, 2  13/
          hc(i) coeff de trans pour les courants chauds   /1   1, 2   1/
          hf(j) coeff de trans pour les courants froids   /1   1, 2   1/
          qc(i) duty du courant i
          qf(j) duty du courant j
          Uij(i,j) coeff global entre i et j
          Uieau(i) coeff global entre i et l utilite froide
          Ujvap(j) coeff global entre j et l utilite chaude;

qc(i)=(tce(i)-tcs(i))*fcpc(i);
qf(j)=(tfs(j)-tfe(j))*fcpf(j);

Uij(i,j)=1/(1/hc(i)+1/hf(j));
Uieau(i)=1/(1/hc(i)+1/heau);
Ujvap(j)=1/(1/hf(j)+1/hvap);

positive variables
          q(i,j,k) puissance echangee entre les courant i et j a l etape k
          eau(i) utilite froide consommee par le courant chaud i
          vap(j) utilite chaude consommee par le courant froid j
          tceetape(i,k) temperature d entre du courant chaud i dans l etape k
          tcsetape(i,k) temperature de sortie du courant chaud i dans l etape k
          tfeetape(j,k) temperature d entre du courant froif j dans l etape k
          tfsetape(j,k) temperature de sortie du courant froid j dans l etape k
          dteijk(i,j,k)  force motrice entre i et j a gauche de k
          dtsijk(i,j,k)  force motrice entre i et j a droite de k
          dtieau(i)  force motrice entre i et l utilite froide
          dtjvap(j)  force motrice entre j et l utilite chaude
          Aq(i,j,k) air des echangeurs procede
          Aeau(i)   air des echangeurs pout utilite froide
          Avap(j)   air des echangeurs pout utilite chaude;

Binary variables
          yijk(i,j,k) couplage des courant i et j a l etapek
          yieau(i) couplage du courant i et de l utilite froide
          yjvap(j) couplage du courant j et de l utilite chaude;
variable
          cout fonction objectif
          entot;

Equations
     bilenerc(i) bilan energie sur les courants chauds
     bilenerf(j) bilan energie sur les courants froids
     biletapc(i,k) bilan sur chaque etape pour chaque courant chaud
     bileau(i) bilan sur les utilites froides
     biletapf(j,k) bilan sur chaque etape pour chaque courant froid
     bilvap(j) bilan sur les utilites chaudes
     connexionc(i,k) pour un chaud la temperature de sortie de k est egale la temperature d entre de k+1
     tentreec(i) temperature d entree des courant chauds
     connexionf(j,k) pour un froid la temperature d entree de k est egale la temperature de sortie de k+1
     tentreef(j) temperature d entree des courant froids
     topoijk(i,j,k) couplage des courant i et j a l etapek
     topoieau(i)    couplage du courant i et de l utilite froide
     topojvap(j)    couplage du courant j et de l utilite chaude
     topoik(i,k) un seul couplage par courant chaud et par etape
     topojk(j,k) un seul couplage par courant froid et par etape
     dteminijk(i,j,k) difference de temperature entre i et j a gauche de k
     dtminieau(i) difference de temperature entre i et l utilite froide a gauche
     dtsminijk(i,j,k) difference de temperature entre i et j a droite de k
     dtminjvap(j) difference de temperature entre j et l utilite chaude a droite
     connexiondt(i,j,k) dtsijk = dteijk+1
     defAq(i,j,k) definition de l air des echangeurs procede
     defAeau(i) definition de l air des echangeurs pout utilite froide
     defAvap(j) definition de l air des echangeurs pout utilite chaude
     fonccout   fonction objectif Cout;

bilenerc(i).. qc(i)=e=sum((j,k),q(i,j,k)) + eau(i);
bilenerf(j).. qf(j)=e=sum((i,k),q(i,j,k)) + vap(j);

biletapc(i,k).. (tceetape(i,k) - tcsetape(i,k))*fcpc(i)=e=sum(j,q(i,j,k));
bileau(i).. (sum(k$(ord(k) eq netape),tcsetape(i,k))-tcs(i))*fcpc(i)=e=eau(i);
biletapf(j,k).. (tfsetape(j,k) - tfeetape(j,k))*fcpf(j)=e=sum(i,q(i,j,k));
bilvap(j).. (tfs(j)-tfsetape(j,'1'))*fcpf(j)=e=vap(j);

connexionc(i,k)$(ord(k) lt netape).. tcsetape(i,k)=e=tceetape(i,k+1);
tentreec(i).. tceetape(i,'1')=e= tce(i);
connexionf(j,k)$(ord(k) lt netape).. tfeetape(j,k)=e=tfsetape(j,k+1);
tentreef(j).. sum(k$(ord(k) eq netape),tfeetape(j,k))=e= tfe(j);

topoijk(i,j,k).. q(i,j,k)=l=yijk(i,j,k)*max(qc(i),qf(j));
topoieau(i).. eau(i)=l= yieau(i)*qc(i);
topojvap(j).. vap(j)=l=yjvap(j)*qf(j);
topoik(i,k).. sum(j,yijk(i,j,k)) =l= 1;
topojk(j,k).. sum(i,yijk(i,j,k)) =l= 1;

dteminijk(i,j,k).. dteijk(i,j,k)=l=tceetape(i,k)-tfsetape(j,k) + dtmax*(1-yijk(i,j,k));
dtminieau(i)..  dtieau(i)=l=sum(k$(ord(k) eq netape),tcsetape(i,k))-teaus + dtmax*(1-yieau(i));
dtsminijk(i,j,k).. dtsijk(i,j,k)=l=tcsetape(i,k)-tfeetape(j,k) + dtmax*(1-yijk(i,j,k));
dtminjvap(j)..  dtjvap(j)=l=tvap-tfsetape(j,'1') + dtmax*(1-yjvap(j));

connexiondt(i,j,k)$(ord(k) lt netape)..  dtsijk(i,j,k)=e=dteijk(i,j,k+1);

defAq(i,j,k)..    q(i,j,k)=e=Uij(i,j)*Aq(i,j,k)*(dteijk(i,j,k)*dtsijk(i,j,k)*(dteijk(i,j,k)+dtsijk(i,j,k))/2)**(1/3);
defAeau(i)..      eau(i)=e=Uieau(i)*Aeau(i)*(dtieau(i)*(tcs(i)-teaue)*(dtieau(i)+(tcs(i)-teaue))/2)**(1/3);
defAvap(j)..      vap(j)=e=Ujvap(j)*Avap(j)*(dtjvap(j)*(tvap-tfs(j))*(dtjvap(j)+(tvap-tfs(j)))/2)**(1/3);

fonccout.. cout=e=
          couteau*sum(i,eau(i)) + coutvap*sum(j,vap(j)) +
          coutfixe*(sum((i,j,k),yijk(i,j,k)) + sum(i,yieau(i)) + sum(j,yjvap(j))) +
          coutaire*(sum((i,j,k),Aq(i,j,k))   + sum(i,Aeau(i))  + sum(j,Avap(j)));



* Bornes

dteijk.lo(i,j,k)=dtmin;
dtsijk.lo(i,j,k)=dtmin;
dtieau.lo(i)=dtmin;
dtjvap.lo(j)=dtmin;

Model HEN/all/ ;

Solve HEN using minlp minimizing cout;


